@@ -23,30 +23,24 @@
 
 namespace OCA\Activity;
 
-use OCP\IConfig;
-use \OCP\User;
-use \OCP\Util;
-use \OC\Files\View;
+use OCP\Activity\IManager;
+use OCP\User;
+use OCP\Util;
+use OC\Files\View;
+
+class ParameterHelper {
+	/** @var \OCP\Activity\IManager */
+	protected $activityManager;
 
-class ParameterHelper
-{
 	/** @var \OC\Files\View */
 	protected $rootView;
 
-	/** @var \OCP\IConfig */
-	protected $config;
-
 	/** @var \OC_L10N */
 	protected $l;
 
-	/**
-	 * @param View $rootView
-	 * @param IConfig $config
-	 * @param \OC_L10N $l
-	 */
-	public function __construct(View $rootView, IConfig $config, \OC_L10N $l) {
+	public function __construct(IManager $activityManager, View $rootView, \OC_L10N $l) {
+		$this->activityManager = $activityManager;
 		$this->rootView = $rootView;
-		$this->config = $config;
 		$this->l = $l;
 	}
 
@@ -136,16 +130,23 @@ class ParameterHelper
 	 * @return string
 	 */
 	protected function prepareUserParam($param, $highlightParams) {
+		// If the username is empty, the action has been performed by a remote
+		// user, or via a public share. We don't know the username in that case
+		if ($param === '') {
+			if ($highlightParams) {
+				return '<strong>' . $this->l->t('"remote user"') . '</strong>';
+			} else {
+				return $this->l->t('"remote user"');
+			}
+		}
+
 		$displayName = User::getDisplayName($param);
 		$param = Util::sanitizeHTML($param);
 		$displayName = Util::sanitizeHTML($displayName);
 
 		if ($highlightParams) {
-			$avatarPlaceHolder = '';
-			if ($this->config->getSystemValue('enable_avatars', true)) {
-				$avatarPlaceHolder = '<div class="avatar" data-user="' . $param . '"></div>';
-			}
-			return $avatarPlaceHolder . '<strong>' . $displayName . '</strong>';
+			return '<div class="avatar" data-user="' . $param . '"></div>'
+				. '<strong>' . $displayName . '</strong>';
 		} else {
 			return $displayName;
 		}
@@ -268,7 +269,7 @@ class ParameterHelper
 				'%s and <strong class="tooltip" title="%s">%n more</strong>',
 				'%s and <strong class="tooltip" title="%s">%n more</strong>',
 				$count - 3,
-				array($firstList, Util::sanitizeHTML($trimmedList)));
+				array($firstList, $trimmedList));
 		}
 		return $this->l->n('%s and %n more', '%s and %n more', $count - 3, array($firstList));
 	}
@@ -287,6 +288,13 @@ class ParameterHelper
 		else if ($app === 'files') {
 			return array(0 => 'file', 1 => 'username');
 		}
+
+		$specialParameters = $this->activityManager->getSpecialParameterList($app, $text);
+
+		if ($specialParameters !== false) {
+			return $specialParameters;
+		}
+
 		return array();
 	}
 }
