@@ -22,34 +22,33 @@
 
 namespace OCA\Activity\Tests;
 
-class ParameterHelperTest extends \PHPUnit_Framework_TestCase {
+class ParameterHelperTest extends TestCase {
 	/** @var string */
 	protected $originalWEBROOT;
 	/** @var \OCA\Activity\ParameterHelper */
 	protected $parameterHelper;
 	/** @var \OC\Files\View */
 	protected $view;
-	/** @var \PHPUnit_Framework_MockObject_MockObject */
-	protected $config;
 
-	public function setUp() {
+	protected function setUp() {
 		parent::setUp();
+
 		$this->originalWEBROOT =\OC::$WEBROOT;
 		\OC::$WEBROOT = '';
 		$l = \OCP\Util::getL10N('activity');
 		$this->view = new \OC\Files\View('');
-
-		$this->config = $this->getMockBuilder('OCP\IConfig')
-			->disableOriginalConstructor()
-			->getMock();
+		$manager = $this->getMock('\OCP\Activity\IManager');
+		$manager->expects($this->any())
+			->method('getSpecialParameterList')
+			->will($this->returnValue(false));
 		$this->parameterHelper = new \OCA\Activity\ParameterHelper(
+			$manager,
 			$this->view,
-			$this->config,
 			$l
 		);
 	}
 
-	public function tearDown() {
+	protected function tearDown() {
 		\OC::$WEBROOT = $this->originalWEBROOT;
 		parent::tearDown();
 	}
@@ -107,6 +106,14 @@ class ParameterHelperTest extends \PHPUnit_Framework_TestCase {
 				'<div class="avatar" data-user="U&lt;ser&gt;A"></div><strong>U&lt;ser&gt;A</strong>',
 				'<strong>/foo/bar.file</strong>',
 			)),
+			array(array('', '/foo/bar.file'), array(0 => 'username'), true, true, array(
+				'<strong>"remote user"</strong>',
+				'<strong>/foo/bar.file</strong>',
+			)),
+			array(array('', '/foo/bar.file'), array(0 => 'username'), true, false, array(
+				'"remote user"',
+				'/foo/bar.file',
+			)),
 
 			array(array('UserA', '/foo/bar.file'), array(0 => 'username', 1 => 'file'), true, true, array(
 				'<div class="avatar" data-user="UserA"></div><strong>UserA</strong>',
@@ -116,25 +123,16 @@ class ParameterHelperTest extends \PHPUnit_Framework_TestCase {
 				'<div class="avatar" data-user="UserA"></div><strong>UserA</strong>',
 				'<a class="filename tooltip" href="/index.php/apps/files?dir=%2Ftmp%2Ftest" title="in tmp">test</a>',
 			), 'tmp/test/'),
-
-			array(array('UserA', '/foo/bar.file'), array(0 => 'username'), true, true, array(
-				'<strong>UserA</strong>',
-				'<strong>/foo/bar.file</strong>',
-			), '', false),
 		);
 	}
 
 	/**
 	 * @dataProvider prepareParametersData
 	 */
-	public function testPrepareParameters($params, $filePosition, $stripPath, $highlightParams, $expected, $createFolder = '', $enableAvatars = true) {
+	public function testPrepareParameters($params, $filePosition, $stripPath, $highlightParams, $expected, $createFolder = '') {
 		if ($createFolder !== '') {
 			$this->view->mkdir('/' . \OCP\User::getUser() . '/files/' . $createFolder);
 		}
-		$this->config->expects($this->any())
-			->method('getSystemValue')
-			->with('enable_avatars', true)
-			->willReturn($enableAvatars);
 		$this->assertEquals(
 			$expected,
 			$this->parameterHelper->prepareParameters($params, $filePosition, $stripPath, $highlightParams)
@@ -142,47 +140,10 @@ class ParameterHelperTest extends \PHPUnit_Framework_TestCase {
 	}
 
 	public function prepareArrayParametersData() {
-		$en = \OCP\Util::getL10N('activity', 'en');
 		return array(
 			array(array(), 'file', true, true, ''),
 			array(array('A/B.txt', 'C/D.txt'), 'file', true, false, 'B.txt and D.txt'),
 			array(array('A/B.txt', 'C/D.txt'), '', true, false, 'A/B.txt and C/D.txt'),
-			array(
-				array('A/B.txt', 'C/D.txt', 'E/F.txt', 'G/H.txt', 'I/J.txt', 'K/L.txt', 'M/N.txt'), 'file', true, false,
-				(string) $en->n('%s and %n more', '%s and %n more', 4, array(implode((string) $en->t(', '), array('B.txt', 'D.txt', 'F.txt'))))
-			),
-			array(
-				array('A/B.txt', 'C/D.txt', 'E/F.txt', 'G/H.txt', 'I/J.txt', 'K/L.txt', 'M/N.txt'), 'file', true, true,
-				(string) $en->n(
-					'%s and <strong class="tooltip" title="%s">%n more</strong>',
-					'%s and <strong class="tooltip" title="%s">%n more</strong>',
-					4,
-					array(
-						implode((string) $en->t(', '), array(
-							'<a class="filename tooltip" href="/index.php/apps/files?dir=%2FA&scrollto=B.txt" title="in A">B.txt</a>',
-							'<a class="filename tooltip" href="/index.php/apps/files?dir=%2FC&scrollto=D.txt" title="in C">D.txt</a>',
-							'<a class="filename tooltip" href="/index.php/apps/files?dir=%2FE&scrollto=F.txt" title="in E">F.txt</a>',
-						)),
-						'G/H.txt, I/J.txt, K/L.txt, M/N.txt',
-					)
-				),
-			),
-			array(
-				array('A"><h1>/B.txt"><h1>', 'C"><h1>/D.txt"><h1>', 'E"><h1>/F.txt"><h1>', 'G"><h1>/H.txt"><h1>', 'I"><h1>/J.txt"><h1>', 'K"><h1>/L.txt"><h1>', 'M"><h1>/N.txt"><h1>'), 'file', true, true,
-				(string) $en->n(
-					'%s and <strong class="tooltip" title="%s">%n more</strong>',
-					'%s and <strong class="tooltip" title="%s">%n more</strong>',
-					4,
-					array(
-						implode((string) $en->t(', '), array(
-							'<a class="filename tooltip" href="/index.php/apps/files?dir=%2FA%22%3E%3Ch1%3E&scrollto=B.txt%22%3E%3Ch1%3E" title="in A&quot;&gt;&lt;h1&gt;">B.txt&quot;&gt;&lt;h1&gt;</a>',
-							'<a class="filename tooltip" href="/index.php/apps/files?dir=%2FC%22%3E%3Ch1%3E&scrollto=D.txt%22%3E%3Ch1%3E" title="in C&quot;&gt;&lt;h1&gt;">D.txt&quot;&gt;&lt;h1&gt;</a>',
-							'<a class="filename tooltip" href="/index.php/apps/files?dir=%2FE%22%3E%3Ch1%3E&scrollto=F.txt%22%3E%3Ch1%3E" title="in E&quot;&gt;&lt;h1&gt;">F.txt&quot;&gt;&lt;h1&gt;</a>',
-						)),
-						'G&quot;&gt;&lt;h1&gt;/H.txt&quot;&gt;&lt;h1&gt;, I&quot;&gt;&lt;h1&gt;/J.txt&quot;&gt;&lt;h1&gt;, K&quot;&gt;&lt;h1&gt;/L.txt&quot;&gt;&lt;h1&gt;, M&quot;&gt;&lt;h1&gt;/N.txt&quot;&gt;&lt;h1&gt;',
-					)
-				),
-			),
 		);
 	}
 
