@@ -28,11 +28,10 @@ class OC_Mail {
 	 * @param string $ccaddress
 	 * @param string $ccname
 	 * @param string $bcc
-	 * @param string $replyTo
 	 * @throws Exception
 	 */
 	public static function send($toaddress, $toname, $subject, $mailtext, $fromaddress, $fromname,
-		$html=0, $altbody='', $ccaddress='', $ccname='', $bcc='', $replyTo='') {
+		$html=0, $altbody='', $ccaddress='', $ccname='', $bcc='') {
 
 		$SMTPMODE = OC_Config::getValue( 'mail_smtpmode', 'sendmail' );
 		$SMTPHOST = OC_Config::getValue( 'mail_smtphost', '127.0.0.1' );
@@ -72,22 +71,21 @@ class OC_Mail {
 		$mailo->From = $fromaddress;
 		$mailo->FromName = $fromname;;
 		$mailo->Sender = $fromaddress;
+		$mailo->XMailer = ' ';
 		try {
 			$toaddress = self::buildAsciiEmail($toaddress);
 			$mailo->AddAddress($toaddress, $toname);
 
-			if($ccaddress<>'') $mailo->AddCC($ccaddress, $ccname);
-			if($bcc<>'') $mailo->AddBCC($bcc);
+			if($ccaddress != '') $mailo->AddCC($ccaddress, $ccname);
+			if($bcc != '') $mailo->AddBCC($bcc);
 
-			if($replyTo !== '') {
-				$mailo->addReplyTo($replyTo);
-			}
+			$mailo->AddReplyTo($fromaddress, $fromname);
 
-			$mailo->WordWrap = 50;
-			if($html==1) $mailo->IsHTML(true); else $mailo->IsHTML(false);
+			$mailo->WordWrap = 78;
+			$mailo->IsHTML($html == 1);
 
 			$mailo->Subject = $subject;
-			if($altbody=='') {
+			if($altbody == '') {
 				$mailo->Body    = $mailtext.OC_MAIL::getfooter();
 				$mailo->AltBody = '';
 			}else{
